--[[
Author: Massiner of Nathrezim
Date: 03-May-2009

Hacks and other notes:
1. The fade effect for the bar background is a good canidate to move into the new animation model (I've done a prototype but it didn't work as the onupdate implementation here).
2. This file could do with some more commenting and general cleanup before reaching v1
3. When we change from vertical to horizontal layout we don't have a fixed pivot point.
	a. This is because the anchor point does not stay fixed when the player drags the bar.
	b. It is possible to code around that issue and correct the position - unfortunately this buggers up positioning between sessions (I tried several different ways to resolve this without luck)
]]


local ExtraBar_Frame = CreateFrame("FRAME"); -- Need a frame to respond to events
ExtraBar_Frame:RegisterEvent("ADDON_LOADED"); -- Fired when saved variables are loaded
ExtraBar_Frame:RegisterEvent("PLAYER_ENTERING_WORLD");

function ExtraBar_Frame:OnEvent(event, arg1)
	if (event == "ADDON_LOADED" and arg1 == "ExtraBar") then
		EBMethods_UpdateSavedData();
		EBMethods_InitBar(ExtraBar1, 1);
		EBMethods_InitBar(ExtraBar2, 2);
		EBMethods_InitBar(ExtraBar3, 3);
		EBMethods_InitBar(ExtraBar4, 4);
				
		--Setup the Config options pages
		EBCMethods_Setup();
		
	elseif (event == "PLAYER_ENTERING_WORLD") then
		ExtraBar_Update();
	end
end
ExtraBar_Frame:SetScript("OnEvent", ExtraBar_Frame.OnEvent);


--This is called when the ExtraBar1 is loaded
function EBMethods_InitBar(self, number)
	local name = self:GetName();
	_G["BINDING_HEADER_"..name] = "Extra Bar "..number;
	_G["BINDING_NAME_CLICK "..name.."Button1:LeftButton"] = name.." Button 1";
	_G["BINDING_NAME_CLICK "..name.."Button2:LeftButton"] = name.." Button 2";
	_G["BINDING_NAME_CLICK "..name.."Button3:LeftButton"] = name.." Button 3";
	_G["BINDING_NAME_CLICK "..name.."Button4:LeftButton"] = name.." Button 4";
	_G["BINDING_NAME_CLICK "..name.."Button5:LeftButton"] = name.." Button 5";
	_G["BINDING_NAME_CLICK "..name.."Button6:LeftButton"] = name.." Button 6";
	_G["BINDING_NAME_CLICK "..name.."Button7:LeftButton"] = name.." Button 7";
	_G["BINDING_NAME_CLICK "..name.."Button8:LeftButton"] = name.." Button 8";
	_G["BINDING_NAME_CLICK "..name.."Button9:LeftButton"] = name.." Button 9";
	_G["BINDING_NAME_CLICK "..name.."Button10:LeftButton"] = name.." Button 10";
	_G["BINDING_NAME_CLICK "..name.."Button11:LeftButton"] = name.." Button 11";
	_G["BINDING_NAME_CLICK "..name.."Button12:LeftButton"] = name.." Button 12";

	self.timing = 0;
	self.vertical = false;
	self.numberOfButtons = 12;
	_G[name.."Number"]:SetAlpha(0);
	_G[name.."Number"]:SetText(number);
	_G[name.."ConfigNumber"]:SetAlpha(0);
	_G[name.."ConfigNumber"]:SetText(number);
	
	self.dragstate = "notshown";
	self:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = "", 
					tile = true, tileSize = 16, edgeSize = 16, 
					insets = { left = 4, right = 4, top = 4, bottom = 4 }});
	self:SetBackdropColor(0.1, 0.1, 0.3, 0);
	
	--This will allow the bar to be dragged, note that EnableMouse must be true for this to work, and is toggled off to lock the bar
	self:SetMovable(true);
	self:SetClampRectInsets(18, -18, -18, 18);
	self:SetClampedToScreen(true);
	
	--This will setup dragging for the bar.
	self:SetScript("OnMouseDown",function() self:StartMoving() end);
	self:SetScript("OnMouseUp",function() self:StopMovingOrSizing() end);
	
	--Setup secure handlers for enabling and disabling the bar, the main driver for doing this is for vehicles where the user stands a good chance of entering/exiting a vehicle while in combat
	self:Execute([[ebButtons = newtable(); owner:GetChildList(ebButtons);]]);		--Get secure references to the buttons on this bar
	self:SetAttribute("_onshow", [[for k, v in ipairs(ebButtons) do v:Enable(); end]]);	--When the bar is shown enable the buttons
	self:SetAttribute("_onhide", [[for k, v in ipairs(ebButtons) do v:Disable(); end]]);	--When the bar is hidden disable them
end

function ExtraBar_Update()
	ABTMethods_LoadAll(ExtraBar_ButtonEntries, ExtraBar_ButtonSettings);	--Make sure the buttons are upto date
	EBCMethods_Update(); -- Apply the configuration settings
end



function ExtraBar_OnEnter(self)
	if (self.dragstate == "notshown" or self.dragstate == "pretiming") then
		self.dragstate = "entered";
	else
		self.dragstate = "fadein";
	end

end

function ExtraBar_OnLeave(self)
	self.dragstate = "left";
end

function ExtraBar_OnUpdate(self, elapsed)
	local dragstate = self.dragstate;
	if (dragstate == "entered") then
		self.pretiming = 0;
		self.dragstate = "pretiming";
	elseif (dragstate == "pretiming") then
		local preTiming = self.pretiming + elapsed;
		self.pretiming = preTiming ;
		if (preTiming > 0.3) then
			self.dragstate = "fadein";
			self.timing = 0;
		end
	elseif (dragstate == "fadein") then
		local fade = self.timing + elapsed * 2;
		if (fade >= 1) then
			fade = 1;
			self.dragstate = "shown";
		end
		self.timing = fade;
		_G[self:GetName().."Number"]:SetAlpha(fade);
		self:SetBackdropColor(0.1, 0.1, 0.3, 0.7 * fade);
	elseif (dragstate == "left") then
		local fade = self.timing - elapsed * 2;
		if (fade <= 0) then
			fade = 0;

			self.dragstate = "notshown";
		end
		self.timing = fade;
		_G[self:GetName().."Number"]:SetAlpha(fade);
		self:SetBackdropColor(0.1, 0.1, 0.3, 0.7 * fade);
	end

end



function EBMethods_SetEnabled(bar, enabled, hideInVehicle)
	if (InCombatLockdown()) then
		return false;
	end
	
	if (not enabled) then
		UnregisterStateDriver(bar, "visibility");
		bar:Hide();
	elseif (hideInVehicle) then
		UnregisterStateDriver(bar, "visibility");
		RegisterStateDriver(bar, "visibility", "[target=vehicle, exists] hide; show" );
	else
		UnregisterStateDriver(bar, "visibility");
		bar:Show();
	end
	return true;
end



function EBMethods_SetAlwaysShowGrid(bar, value)
	local buttons = {bar:GetChildren()};
	for i,v in ipairs(buttons) do
		ABTMethods_SetAlwaysShowGrid(v, value);
	end
	return true;
end



function EBMethods_SetLockBar(bar, lock)
	if (InCombatLockdown()) then
		return false;
	end
	
	bar:EnableMouse(not lock);
	return true;
end



function EBMethods_SetLockButtons(bar, value)
	local buttons = {bar:GetChildren()};
	for i,v in ipairs(buttons) do
		ABTMethods_SetLockButton(v, value);
	end
	return true;
end



function EBMethods_SetVertical(bar, value)
	if (InCombatLockdown()) then
		return false;
	end
	
	local buttons ={bar:GetChildren()};
	local prev;
	bar.vertical = value;
	if (value) then
		bar:SetWidth(72);
		bar:SetHeight(bar.numberOfButtons * 42 + 30);		
	else
		bar:SetHeight(72);
		bar:SetWidth(bar.numberOfButtons * 42 + 30);		
	end

	for i,v in ipairs(buttons) do
		if (i ~= 1) then
			if (value) then
				v:SetPoint("TOPLEFT", prev, "BOTTOMLEFT", 0, -6);
			else
				v:SetPoint("TOPLEFT", prev, "TOPRIGHT", 6, 0);
			end
		end
		prev = v;
	end
	return true;
end

function EBMethods_SetDisableTooltips(bar, value)
	local buttons = {bar:GetChildren()};
	for i,v in ipairs(buttons) do
		ABTMethods_SetDisableTooltip(v, value);
	end
	return true;
end

function EBMethods_SetNumberOfButtons(bar, value)
	if (InCombatLockdown()) then
		return false;
	end
	local buttons ={bar:GetChildren()};
	bar.numberOfButtons = value;
	for i,v in ipairs(buttons) do
		if (i <= value) then
			ABTMethods_SetEnabled(v, true);
		else
			ABTMethods_SetEnabled(v, false);
		end
	end
	if (bar.vertical) then
		bar:SetWidth(72);
		bar:SetHeight(bar.numberOfButtons * 42 + 30);		
	else
		bar:SetHeight(72);
		bar:SetWidth(bar.numberOfButtons * 42 + 30);		
	end
	
	return true;
end



function EBMethods_UpdateSavedData()

	if (not ExtraBar_ButtonEntries) then
		ExtraBar_ButtonEntries = {};
		ExtraBar_ButtonSettings = {};
		ExtraBar_Config = {};
	end
	
	if (not ExtraBar_Config["version"] or ExtraBar_Config["version"]+0 < 0.6) then
		ExtraBar_Config = {};
		ExtraBar_Config["version"] = 0.6;
	end
	
	ExtraBar_ButtonEntries, ExtraBar_ButtonSettings = ABTMethods_UpdateSavedDataVersion(ExtraBar_ButtonEntries, ExtraBar_ButtonSettings);
end

