--[[
Author: Massiner of Nathrezim
Date: 03-May-2009
Acknowledgements: I based my implementation of EBCMethods_SetupButton on the one I saw in DruidBar (when looking at how other mods handled integration with the addons setting ui)

Hacks and other notes:
1. Several of the settings cannot be changed while in combat, the ui will block if a setting can't be changed due to this.
2. The settings are applied when the option is changed although cancel will roll it back
2. default, and cancel options will not be able to alter the settings back if the player is in combat
]]

function EBCMethods_Setup()

	EBCMethods_SetupBar(EB1Cfg, ExtraBar1, "Extra Bar 1", "Configure the behaviour of the 1st Extra Bar");
	EBCMethods_SetupBar(EB2Cfg, ExtraBar2, "Extra Bar 2", "Configure the behaviour of the 2nd Extra Bar");
	EBCMethods_SetupBar(EB3Cfg, ExtraBar3, "Extra Bar 3", "Configure the behaviour of the 3rd Extra Bar");
	EBCMethods_SetupBar(EB4Cfg, ExtraBar4, "Extra Bar 4", "Configure the behaviour of the 4th Extra Bar");
end

function EBCMethods_ShowBarNums(self, value)
	_G[EB1Cfg.bar:GetName().."ConfigNumber"]:SetAlpha(value);
	_G[EB2Cfg.bar:GetName().."ConfigNumber"]:SetAlpha(value);
	_G[EB3Cfg.bar:GetName().."ConfigNumber"]:SetAlpha(value);
	_G[EB4Cfg.bar:GetName().."ConfigNumber"]:SetAlpha(value);
end

function EBCMethods_SetupBar(self, bar, title, subText)
	local name = self:GetName();
	self.name = title;
	self.bar = bar;
	_G[name.."Title"]:SetText(title);
	_G[name.."SubText"]:SetText(subText);
	--"Configure the behaviour of the Extra Bar"
	
	EBCMethods_SetupButton(_G[name.."ButtonEnabled"],	"Enable Bar",
							name.."Enabled", true, EBCMethods_UpdateEnabled,
							"Enable this Bar");

	EBCMethods_SetupButton(_G[name.."ButtonAlwaysShowGrid"], "Always Show Grid",
							name.."AlwaysShowGrid", true, EBCMethods_UpdateAlwaysShowGrid,
							"Always display this bars grid");

	EBCMethods_SetupButton(_G[name.."ButtonLockButtons"], "Lock Buttons",
							name.."LockButtons", false, EBCMethods_UpdateLockButtons,
							"Check this option to prevent\n"..
							"actions being dragged from the bar\n"..
							"(you will still be able to drop actions on the bar)");

	EBCMethods_SetupButton(_G[name.."ButtonLockBar"], "Lock Bar",
							name.."LockBar", false, EBCMethods_UpdateLockBar,
							"Check this option to prevent\n"..
							"the bar being moved");

	EBCMethods_SetupButton(_G[name.."ButtonHideInVehicle"], "Hide on Enter Vehicle",
							name.."HideInVehicle", true, EBCMethods_UpdateHideInVehicle,
							"Check this option to hide the bar\n"..
							"when the player enters a vehicle");

	EBCMethods_SetupButton(_G[name.."ButtonVertical"], "Vetical",
							name.."Vertical", false, EBCMethods_UpdateVertical,
							"Lay the bar out vertically\n"..
							"rather than horizontally");

	EBCMethods_SetupButton(_G[name.."ButtonDisableTooltips"], "Disable Tooltips",
							name.."DisableTooltips", false, EBCMethods_UpdateDisableTooltips,
							"Check to disable tooltips\n"..
							"for buttons on this bar");
	
	EBCMethods_SetupSlider(_G[name.."SliderNumber"], "Number of Buttons",
							name.."NumberOFButtons", 12, EBCMethods_UpdateNumberOfButtons,
							"Adjust the number of buttons\n"..
							"on the bar", 1, 12, 1);

							
	self.ShowBarNums = EBCMethods_ShowBarNums;
	self:SetAttribute("_onshow", [[control:CallMethod("ShowBarNums", 1);]]);
	self:SetAttribute("_onhide", [[control:CallMethod("ShowBarNums", 0);]]);
	

	self.okay = EBCMethods_Okay;
    self.cancel = EBCMethods_Cancel;
	self.default = EBCMethods_Default;
	self.parent = "Extra Bar";

	InterfaceOptions_AddCategory(self);
end



function EBCMethods_ButtonClick(self)
	if (self:GetChecked()) then
		PlaySound("igMainMenuOptionCheckBoxOn");
	else
		PlaySound("igMainMenuOptionCheckBoxOff");
	end
end



function EBCMethods_SetupSlider(self, name, setting, default, func, tooltip, minv, maxv, step)
	_G[self:GetName().."Text"]:SetText(name);
	_G[self:GetName().."Low"]:SetText(minv);
	_G[self:GetName().."High"]:SetText(maxv);
	self.setting = setting;
	self.func = func;
	self.default = default;
	self:SetMinMaxValues(minv, maxv);
	self:SetValueStep(step);
	if (type(ExtraBar_Config[setting]) == "nil") then
		ExtraBar_Config[setting] = default;
	end
	
	self:SetValue(ExtraBar_Config[setting]);
	_G[self:GetName().."Value"]:SetText(self:GetValue());	
	self:SetScript("OnValueChanged", function() if (not self:func()) then self:SetValue(_G[self:GetName().."Value"]:GetText()); end _G[self:GetName().."Value"]:SetText(self:GetValue()); end)
	self:SetScript("OnEnter", function() GameTooltip:SetOwner(self, "ANCHOR_RIGHT"); GameTooltip:SetText(tooltip); GameTooltip:Show(); end);
	self:SetScript("OnLeave", function() GameTooltip:Hide(); end);
	
end

function EBCMethods_SetupButton(self, name, setting, default, func, tooltip)	--This function is based off of how the DruidBar mod worked
	_G[self:GetName().."Text"]:SetText(name);
	self.setting = setting;
	self.func = func;
	self.default = default;

	if (type(ExtraBar_Config[setting]) == "nil") then
		ExtraBar_Config[setting] = default;
	end

	self:SetChecked(ExtraBar_Config[setting]);
	self:SetScript("OnClick", function() EBCMethods_ButtonClick(self); if (not self:func()) then self:SetChecked(not self:GetChecked()); end end);
	self:SetScript("OnEnter", function() GameTooltip:SetOwner(self, "ANCHOR_RIGHT"); GameTooltip:SetText(tooltip); GameTooltip:Show(); end);
	self:SetScript("OnLeave", function() GameTooltip:Hide(); end);
end


function EBCMethods_Update()
	EBCMethods_UpdateBar(EB1Cfg);
	EBCMethods_UpdateBar(EB2Cfg);
	EBCMethods_UpdateBar(EB3Cfg);
	EBCMethods_UpdateBar(EB4Cfg);

end

function EBCMethods_UpdateBar(self)
	local controls = {self:GetChildren()};
	for i,v in ipairs(controls) do
		v:func();
	end
end

function EBCMethods_UpdateEnabled(self)
	local button = _G[self:GetParent():GetName().."ButtonHideInVehicle"];
	return EBMethods_SetEnabled(self:GetParent().bar, self:GetChecked(), button:GetChecked());
end

function EBCMethods_UpdateAlwaysShowGrid(self)
	return EBMethods_SetAlwaysShowGrid(self:GetParent().bar, self:GetChecked());
end

function EBCMethods_UpdateLockButtons(self)
	return EBMethods_SetLockButtons(self:GetParent().bar, self:GetChecked());
end

function EBCMethods_UpdateLockBar(self)
	return EBMethods_SetLockBar(self:GetParent().bar, self:GetChecked());
end

function EBCMethods_UpdateHideInVehicle(self)
	local button = _G[self:GetParent():GetName().."ButtonEnabled"];
	return EBMethods_SetEnabled(self:GetParent().bar, button:GetChecked(), self:GetChecked());
end

function EBCMethods_UpdateVertical(self)
	return EBMethods_SetVertical(self:GetParent().bar, self:GetChecked());
end

function EBCMethods_UpdateDisableTooltips(self)
	return EBMethods_SetDisableTooltips(self:GetParent().bar, self:GetChecked());
end

function EBCMethods_UpdateNumberOfButtons(self)
	return EBMethods_SetNumberOfButtons(self:GetParent().bar, self:GetValue());
end


function EBCMethods_Okay(self)

	local controls = {self:GetChildren()};
	for i,v in ipairs(controls) do
		local objectType = v:GetObjectType();
		if (objectType == "CheckButton") then
			ExtraBar_Config[v.setting] = v:GetChecked() ~= nil; -- We do this to ensure we get a false/true as opposed to 1/nil value (the nil value causes the button to use the default at setup)
		elseif (objectType == "Slider") then
			ExtraBar_Config[v.setting] = v:GetValue();
		end
	end
end

function EBCMethods_Cancel(self)
	if (InCombatLockdown()) then
		--we prevent a cancel and will force an okay so that our config stays in sync with the current state
		EBCMethods_Okay(self);
		return;
	end
	
	--Set the buttons back to saved settings, and update the bar
	local controls = {self:GetChildren()};
	for i,v in ipairs(controls) do
		local objectType = v:GetObjectType();
		if (objectType == "CheckButton") then
			v:SetChecked(ExtraBar_Config[v.setting]);
		elseif (objectType == "Slider") then
			v:SetValue(ExtraBar_Config[v.setting]);
		end
		v:func();
	end
end


function EBCMethods_Default(self)
	if (InCombatLockdown()) then
		--we prevent a change to default
		return;
	end
	
	--Otherwise we can update the state to default values
	local controls = {self:GetChildren()};
	for i,v in ipairs(controls) do
		local objectType = v:GetObjectType();
		if (objectType == "CheckButton") then
			v:SetChecked(v.default);
		elseif (objectType == "Slider") then
			v:SetValue(v.default);
		end
		v:func();
	end
	
	--Save the defaults
	EBCMethods_Okay(self);
end
